@@ -3,6 +3,9 @@ import hashlib
 import logging
 import tempfile
 
+import aiohttp
+import aiohttp_socks
+
 import synapse.exc as s_exc
 import synapse.common as s_common
 
@@ -211,6 +214,10 @@ class AxonApi(s_cell.CellApi, s_share.Share):  # type: ignore
         await self._reqUserAllowed(('axon', 'upload'))
         return await UpLoadShare.anit(self.cell, self.link)
 
+    async def wget(self, url, params=None, headers=None, json=None, body=None, method='GET', ssl=True):
+        await self._reqUserAllowed(('axon', 'wget'))
+        return await self.cell.wget(url, params=params, headers=headers, json=json, body=body, method=method, ssl=ssl)
+
     async def metrics(self):
         await self._reqUserAllowed(('axon', 'has'))
         return await self.cell.metrics()
@@ -231,7 +238,11 @@ class Axon(s_cell.Cell):
             'type': 'integer',
             'minimum': 1,
             'hidecmdl': True,
-        }
+        },
+        'http:proxy': {
+            'description': 'An aiohttp-socks compatible proxy URL to use in the wget API.',
+            'type': 'string',
+        },
     }
 
     async def __anit__(self, dirn, conf=None):  # type: ignore
@@ -368,3 +379,50 @@ class Axon(s_cell.Cell):
         Given a list of sha256 bytes, returns a list of the hashes we want bytes for.
         '''
         return [s for s in sha256s if not await self.has(s)]
+
+    async def wget(self, url, params=None, headers=None, json=None, body=None, method='GET', ssl=True):
+        '''
+        Stream a file download directly into the axon.
+        '''
+        connector = None
+        proxyurl = self.conf.get('http:proxy')
+        if proxyurl is not None:
+            connector = aiohttp_socks.ProxyConnector.from_url(proxyurl)
+
+        async with aiohttp.ClientSession(connector=connector) as sess:
+
+            try:
+
+                async with sess.request(method, url, headers=headers, params=params, json=json, data=body, ssl=ssl) as resp:
+
+                    info = {
+                        'ok': True,
+                        'code': resp.status,
+                        'headers': dict(resp.headers),
+                    }
+
+                    hashset = s_hashset.HashSet()
+
+                    async with await self.upload() as upload:
+
+                        byts = await resp.content.read(CHUNK_SIZE)
+                        while byts:
+                            await upload.write(byts)
+                            hashset.update(byts)
+                            byts = await resp.content.read(CHUNK_SIZE)
+
+                        size, _ = await upload.save()
+
+                    info['size'] = size
+                    info['hashes'] = dict([(n, s_common.ehex(h)) for (n, h) in hashset.digests()])
+
+                    return info
+
+            except asyncio.CancelledError:
+                raise
+
+            except Exception as e:
+                return {
+                    'ok': False,
+                    'mesg': str(e),
+                }
