@@ -128,7 +128,7 @@ async def getAhaProxy(urlinfo):
     for ahaurl, cnfo in list(aha_clients.items()):
         client = cnfo.get('client')
         try:
-            proxy = await client.proxy(timeout=1)
+            proxy = await client.proxy(timeout=10)
 
             ahasvc = await proxy.getAhaSvc(host)
             if ahasvc is None:
@@ -291,7 +291,6 @@ class Genr(Share):
     async def __aiter__(self):
 
         try:
-
             while not self.isfini:
 
                 for retn in await self.queue.slice():
@@ -300,6 +299,8 @@ class Genr(Share):
 
                     yield s_common.result(retn)
 
+            raise s_exc.LinkShutDown(mesg='Remote peer disconnected')
+
         finally:
             await self.fini()
 
@@ -606,9 +607,11 @@ class Proxy(s_base.Base):
 
                         mesg = await link.rx()
                         if mesg is None:
-                            return
+                            raise s_exc.LinkShutDown(mesg=mesg)
 
-                        assert mesg[0] == 't2:yield'
+                        if mesg[0] != 't2:yield':  # pragma: no cover
+                            info = 'Telepath protocol violation:  unexpected message received'
+                            raise s_exc.BadMesgFormat(mesg=info)
 
                         retn = mesg[1].get('retn')
                         if retn is None:
